import { assocPath } from './assocPath'

test('adds a key to an empty object', () => {
  expect(assocPath('a', 1, {})).toEqual({ a : 1 })
})

test('adds a key to a non-empty object', () => {
  expect(assocPath('b', 2, { a : 1 })).toEqual({
    a : 1,
    b : 2,
  })
})

test('adds a nested key to a non-empty object', () => {
  expect(assocPath('b.c', 2, { a : 1 })).toEqual({
    a : 1,
    b : {
      c : 2
    }
  })
})

test('adds a key to a non-empty object - curry case 1', () => {
  expect(assocPath('b', 2)({ a : 1 })).toEqual({
    a : 1,
    b : 2,
  })
})

test('adds a nested key to a non-empty object - curry case 1', () => {
  expect(assocPath('b.c', 2)({ a : 1 })).toEqual({
    a : 1,
    b : {
      c : 2
    },
  })
})

test('adds a nested array to a non-empty object - curry case 1', () => {
  expect(assocPath('b.0', 2)({ a : 1 })).toEqual({
    a : 1,
    b : [
      2
    ]
  })
})

test('adds a key to a non-empty object - curry case 2', () => {
  expect(assocPath('b')(2, { a : 1 })).toEqual({
    a : 1,
    b : 2,
  })
})

test('adds a key to a non-empty object - curry case 3', () => {
  const result = assocPath('b')(2)({ a : 1 })

  expect(result).toEqual({
    a : 1,
    b : 2,
  })
})

test('changes an existing key', () => {
  expect(assocPath('a', 2, { a : 1 })).toEqual({ a : 2 })
})

test('undefined is considered an empty object', () => {
  expect(assocPath('a', 1, undefined)).toEqual({ a : 1 })
})

test('null is considered an empty object', () => {
  expect(assocPath('a', 1, null)).toEqual({ a : 1 })
})

test('value can be null', () => {
  expect(assocPath('a', null, null)).toEqual({ a : null })
})

test('value can be undefined', () => {
  expect(assocPath('a', undefined, null)).toEqual({ a : undefined })
})

test('assignment is shallow', () => {
  expect(assocPath('a', { b : 2 }, { a : { c : 3 } })).toEqual({ a : { b : 2 } })
})
